--[[
PansyLabs Timers. Refer to LICENSE and README.md for details

This file contains object pool related functionality.
]]--

function PL_Timers:InitialisePools() 
  self.timerPool = ZO_ObjectPool:New(function() return self:CreateTimerBinding() end, function(object, pool) self:ResetTimerBinding(object, pool) end)
  self.infoPool = ZO_ObjectPool:New(function() return self:CreateInfo() end, function(object, pool) self:ResetInfo(object, pool) end)
  self.captionPool = ZO_ObjectPool:New(function() return self:CreateCaption() end, function(object, pool) self:ResetCaption(object, pool) end)
  -- This upvalue & closures are an easy means to work a counter for slot offsets etc. An even better impl. would be something like an 
  -- ObjectPool which can be asked for N items at once, and never returns fragmented ranges.
  local counter = 0
  PL_Timers.PeekSlot = function (self)
    return counter
  end
  PL_Timers.TakeSlot = function(self)
    local oldCounter = counter
    counter = counter + 1
    return oldCounter
  end
  PL_Timers.ReleaseSlot = function(self)
    counter = counter - 1
    return counter
  end
end

function PL_Timers:CreateTimerBinding()
  local binding = {}
  binding.control = ZO_ObjectPool_CreateNamedControl(PL_Timers.name .. "Timer", "PL_Timers_Timer", self.timerPool, self.control)
  return binding
end

function PL_Timers:ResetTimerBinding(binding, pool)
  binding.timerIdx = nil
  binding.timeline = nil
  binding.control:SetHidden(true)
end

function PL_Timers:CreateInfo()
  return ZO_ObjectPool_CreateNamedControl(PL_Timers.name .. "Info", "PL_Timers_InfoText", self.infoPool, self.control)
end

function PL_Timers:ResetInfo(info, pool)
  info:SetHidden(true)
end

function PL_Timers:CreateCaption()
  return ZO_ObjectPool_CreateNamedControl(PL_Timers.name .. "Caption", "PL_Timers_Caption", self.captionPool, self.control)
end

function PL_Timers:ResetCaption(caption, pool)
  caption:SetHidden(true)
end
