--[[
Settings management for PL_Timers.
]]--
local LAM = LibStub("LibAddonMenu-2.0")

PL_Timers.Settings = {}
local n = PL_Timers.name
local S = PL_Timers.Settings
local L = PL_Timers.Strings
S.defaults = {
  ["enabled"] = true,
  ["enableAlarms"] = true,
  ["enableSounds"] = true,
  ["enableTimersOnLogin"] = true,
  ["enableShowAllChars"] = true,
  ["enableAlarmAllChars"] = true,
  ["alarm1Time"] = 60,   -- 1 min
  ["alarm2Time"] = 900,  -- 15 mins
  ["alarm3Time"] = 3600, -- 1 hour
  ["redTime"] = 60,
  ["amberTime"] = 900,
  ["alarmLeadTime"] = 4,
  ["unusedSlotWarning" .. CRAFTING_TYPE_BLACKSMITHING] = 3,
  ["unusedSlotWarning" .. CRAFTING_TYPE_CLOTHIER] = 3,
  ["unusedSlotWarning" .. CRAFTING_TYPE_WOODWORKING] = 3,
  ["sounds"] = {
    alarm = "New_Notification",
    timers = "Backpack_Close",
  },
  ["RAG"] = {
    red = { hex = "C21E1F", rgba = { 0.76, 0.12, 0.12, 0.6 } },
    amber = { hex = "C2C21F", rgba = { 0.76, 0.76, 0.12, 0.6 } },
    green = { hex = "1EC21F", rgba = { 0.12, 0.76, 0.12, 0.6 } },
  }
}

local unusedSlotWarningOpts = {}
unusedSlotWarningOpts[L.unusedSlotWarningOptionNever] = 1
unusedSlotWarningOpts[L.unusedSlotWarningOptionOne] = 2
unusedSlotWarningOpts[L.unusedSlotWarningOptionAll] = 3

local unusedSlotWarningOptsRev = {}
for k, v in pairs(unusedSlotWarningOpts) do
  unusedSlotWarningOptsRev[v] = k
end

-- Addon settings panel data.
local panelData = {
  type = "panel",
  name = L.settingsTitle,
  version = PL_Timers.version,
  author = "PansyLabs (GetBackYouPansy; @iyoro EU megaserver)",
  registerForRefresh = true,
}

-- Addon settings option (controls) data.
local optionData = {
  [1] = {
    type = "checkbox",
    name = L.settingCheckboxEnabled,
    tooltip = L.settingTooltipCheckboxEnabled,
    getFunc = function() return S.vars.enabled end,
    setFunc = function(value) S.vars.enabled = value end,
  },
  [2] = {
    type = "checkbox",
    name = L.settingCheckboxEnableSounds,
    tooltip = L.settingTooltipCheckboxEnableSounds,
    getFunc = function() return S.vars.enableSounds end,
    setFunc = function(value) S.vars.enableSounds = value end,
  },
  [3] = {
    type = "checkbox",
    name = L.settingCheckboxEnableAlarms,
    tooltip = L.settingTooltipCheckboxEnableAlarms,
    getFunc = function() return S.vars.enableAlarms end,
    setFunc = function(value) S.vars.enableAlarms = value end,
  },
  [4] = {
    type = "checkbox",
    name = L.settingCheckboxEnableTimersOnLogin,
    tooltip = L.settingTooltipCheckboxEnableTimersOnLogin,
    getFunc = function() return S.vars.enableTimersOnLogin end,
    setFunc = function(value) S.vars.enableTimersOnLogin = value end,
  },
  [5] = {
    type = "checkbox",
    name = L.settingCheckboxEnableShowAllChars,
    tooltip = L.settingTooltipCheckboxEnableShowAllChars,
    getFunc = function() return S.vars.enableShowAllChars end,
    setFunc = function(value) S.vars.enableShowAllChars = value end,
  },
  [6] = {
    type = "checkbox",
    name = L.settingCheckboxEnableAlarmAllChars,
    tooltip = L.settingTooltipCheckboxEnableAlarmAllChars,
    getFunc = function() return S.vars.enableAlarmAllChars end,
    setFunc = function(value) S.vars.enableAlarmAllChars = value end,
  },
  [7] = {
    type = "editbox",
    name = string.format(L.settingTextBoxAlarmTime, 1),
    tooltip = L.settingTooltipTextBoxAlarmTime,
    getFunc = function() return S.vars.alarm1Time end,
    setFunc = function(value) S.SetTime("alarm1Time", value) end,
  },
  [8] = {
    type = "editbox",
    name = string.format(L.settingTextBoxAlarmTime, 2),
    tooltip = L.settingTooltipTextBoxAlarmTime,
    getFunc = function() return S.vars.alarm2Time end,
    setFunc = function(value) S.SetTime("alarm2Time", value) end,
  },
  [9] = {
    type = "editbox",
    name = string.format(L.settingTextBoxAlarmTime, 3),
    tooltip = L.settingTooltipTextBoxAlarmTime,
    getFunc = function() return S.vars.alarm3Time end,
    setFunc = function(value) S.SetTime("alarm3Time", value) end,
  },
  [10] = {
     type = "editbox",
    name = L.settingTextBoxRAGTimeRed,
    tooltip = L.settingTooltipTextBoxRAGTime,
    getFunc = function() return S.vars.redTime end,
    setFunc = function(value) S.SetTime("redTime", value) end,
  },
  [11] = {
    type = "editbox",
    name = L.settingTextBoxRAGTimeAmber,
    tooltip = L.settingTooltipTextBoxRAGTime,
    getFunc = function() return S.vars.amberTime end,
    setFunc = function(value) S.SetTime("amberTime", value) end,
  },
  [12] = {
  	type = "description",
    text = L.settingDescUnusedSlotWarnings,
  },
  [13] = {
    type = "dropdown",
    name = PL_Timers.Crafts[CRAFTING_TYPE_BLACKSMITHING].name,
    choices = unusedSlotWarningOptsRev,
    getFunc = function() return S.GetUnusedSlotWarning(CRAFTING_TYPE_BLACKSMITHING) end,
    setFunc = function(value) S.SetUnusedSlotWarning(CRAFTING_TYPE_BLACKSMITHING, value) end,
  },
  [14] = {
    type = "dropdown",
    name = PL_Timers.Crafts[CRAFTING_TYPE_CLOTHIER].name,
    choices = unusedSlotWarningOptsRev,
    getFunc = function() return S.GetUnusedSlotWarning(CRAFTING_TYPE_CLOTHIER) end,
    setFunc = function(value) S.SetUnusedSlotWarning(CRAFTING_TYPE_CLOTHIER, value) end,
  },
  [15] = {
    type = "dropdown",
    name = PL_Timers.Crafts[CRAFTING_TYPE_WOODWORKING].name,
    choices = unusedSlotWarningOptsRev,
    getFunc = function() return S.GetUnusedSlotWarning(CRAFTING_TYPE_WOODWORKING) end,
    setFunc = function(value) S.SetUnusedSlotWarning(CRAFTING_TYPE_WOODWORKING, value) end,
  }
}

-- Set a time, validating the given value as an int.
function S.SetTime(key, v)
  local secs = tonumber(v)
  if (secs ~= nil) then
    secs = math.max(1, secs)
    S.vars[key] = secs
  else
    PL_Timers.msg(string.format(L.badSecondsValue, v, S.vars[key]), "ff0000")
  end
end

function S.SetUnusedSlotWarning(craft, v)
  local key = "unusedSlotWarning" .. craft
  S.vars[key] = unusedSlotWarningOpts[v]
end

function S.GetUnusedSlotWarning(craft)
  return unusedSlotWarningOptsRev[S.vars["unusedSlotWarning" .. craft]]
end

function PL_Timers.InitAddonMenu()
  LAM:RegisterAddonPanel("PL_TimersPanel", panelData)  
  LAM:RegisterOptionControls("PL_TimersPanel", optionData)
end

function PL_Timers.InitKeyBindings()
  -- Not obvious how to also set up stringIds for the category and layer.
  ZO_CreateStringId("SI_BINDING_NAME_SHOW_TIMERS", PL_Timers.Strings.keybindShowTimers)
end

-- Only init the saved vars when the addon is loaded, else they do not appear to get wired up properly (e.g. do not persist).
local id = PL_Timers.name .. "OnAddOnLoaded"
EVENT_MANAGER:RegisterForEvent(id, EVENT_ADD_ON_LOADED, function(_, addon)
  if (addon ~= PL_Timers.name) then return end
  EVENT_MANAGER:UnregisterForEvent(id, EVENT_ADD_ON_LOADED)
  S.vars = ZO_SavedVars:New(PL_Timers.name .. "Settings", PL_Timers.settingsVersion, nil, S.defaults, nil)
  PL_Timers.AWT = ZO_SavedVars:NewAccountWide(PL_Timers.name .. "AWT", PL_Timers.version, nil, {data={}})
  setmetatable(S, { __index = S.vars })
  PL_Timers.InitAddonMenu()
  PL_Timers.InitKeyBindings()
end)
