--[[
PansyLabs Timers. Refer to LICENSE and README.md for details

This file contains 'static' members i.e. not instance methods.
]]--

PL_Timers = {}
PL_Timers.version = 3.34
-- Starting in 3.3, record the settings compat. version. The first settingsVersion was 3.22, as that was the 
-- production release at the time. Changing this forces players to re-do all the settings!
PL_Timers.settingsVersion = 3.22
PL_Timers.name = "PL_Timers"

-- This function doesn't seem like it should be necessary - it removes any trailing chars after a ^ in a string.
-- This is used because craft names appear to end in e.g. ^f in German.
function PL_Timers.UndecorateString(input)
  return string.gsub(input, "%^.+", "") 
end

-- Populate the craft details which don't change.
PL_Timers.numTraitItems = GetNumSmithingTraitItems()
PL_Timers.Crafts = {}
PL_Timers.Crafts[CRAFTING_TYPE_BLACKSMITHING] = {}
PL_Timers.Crafts[CRAFTING_TYPE_CLOTHIER] = {}
PL_Timers.Crafts[CRAFTING_TYPE_WOODWORKING] = {}
for craftType, craftData in pairs(PL_Timers.Crafts) do
  craftData["lines"] = GetNumSmithingResearchLines(craftType)
  local skillType, skillIndex = GetCraftingSkillLineIndices(craftType)
  local skillName, _ = GetSkillLineInfo(skillType, skillIndex)
  craftData["name"] = PL_Timers.UndecorateString(skillName)
end

PL_Timers.Strings = {en={}, de={}, fr={}}
local L = PL_Timers.Strings
setmetatable(PL_Timers.Strings, { __index = function(t, k)
    local lang = GetCVar("language.2")
    local str = t[lang][k]
    if str == nil then
      local fallback = t.en[k]
      if fallback == nil then
        local proto = lang .. "." .. k
        --d("|cff0000PL Research Timers: Missing string " .. proto .. ".")
        str = proto
      else
        str = fallback
      end
    end
  return str
end})

-- English localisation
L.en.notUsingAllTimers = "%s: Only using %d out of %d research slots!"
L.en.missingSound = "Research Timers warning: No sound defined '%s'"
L.en.extTimerCompleted = "%s has finished researching %s - %s (%s)"
L.en.extTimerCompletedInfobox = "Finished researching %s - %s (%s)"
L.en.badSecondsValue = "Bad seconds value value '%s'. Keeping old value %d."
L.en.settingsTitle = "Research Timers"
L.en.settingCheckboxEnabled = "Enable research timers"
L.en.settingTooltipCheckboxEnabled = "Enable all functionality"
L.en.settingCheckboxEnableSounds = "Enable sounds"
L.en.settingTooltipCheckboxEnableSounds = "Enable sounds"
L.en.settingCheckboxEnableAlarms = "Enable alarms"
L.en.settingTooltipCheckboxEnableAlarms = "Show alarms/reminders at various times during the research"
L.en.settingCheckboxEnableTimersOnLogin = "Show timers on login"
L.en.settingTooltipCheckboxEnableTimersOnLogin = "Show timers when logging in"
L.en.settingCheckboxEnableShowAllChars = "Full timers from all characters"
L.en.settingTooltipCheckboxEnableShowAllChars = "When timers are shown on login, or manually using the keybind or /timers, this option determines if all timers from characters on this account will be displayed.\n\nThis does not affect alarm timers."
L.en.settingCheckboxEnableAlarmAllChars = "Alarm timers from all characters"
L.en.settingTooltipCheckboxEnableAlarmAllChars = "This option determines whether timers may be shown from characters other than the active character due to alarms.\n\nYou may wish to have this turned ON even if you have the full timers option above turned OFF."
L.en.settingTextBoxAlarmTime = "Alarm %d time (sec)"
L.en.settingTooltipTextBoxAlarmTime = "Alarm time, in number of seconds. \n\nFor one minute before the research ends use the value 60. For one hour use 3600, etc."
L.en.settingTextBoxRAGTimeAmber = "Remaining time for amber (sec)"
L.en.settingTextBoxRAGTimeRed = "Remaining time for red (sec)"
L.en.settingTooltipTextBoxRAGTime = "Remaining time, in number of seconds, at which the timer colour changes."
L.en.settingDescUnusedSlotWarnings = "Show a warning along with timers for this character when there are unused research slots for..."
L.en.unusedSlotWarningOptionNever = "Never (don't care)"
L.en.unusedSlotWarningOptionOne = "One or more unused slots"
L.en.unusedSlotWarningOptionAll = "All slots are unused"
L.en.keybindShowTimers = "Show timers"
-- TODO: French and German translations
-- TODO: Better support for other languages e.g. different arg order.

-- Function to append to the active chat window. Works better than d() which only seems to 
-- post to the first tab.
function PL_Timers.msg(msg, color)
  color = color or "FFFFFF"
  local cmsg = "|c" .. color .. msg
  --ZO_ChatWindow.container.currentBuffer:AddMessage(cmsg)
  CHAT_SYSTEM:AddMessage(cmsg)
end

function PL_Timers.PlaySound(name)
  if (not PL_Timers.Settings.enableSounds) then return end
  local snd = PL_Timers.Settings.sounds[name]
  if (snd == nil) then 
    PL_Timers.msg(string.format(PL_Timers.Strings.missingSound, name), "FFFF00")
    return
  end
  PlaySound(snd)
end
